@@ -1,3 +1,5 @@
+require 'puppet/concurrent/thread_local_singleton'
+
 module Puppet::Pops
 module Types
 # String
@@ -7,6 +9,8 @@ module Types
 # @api public
 #
 class TypeFormatter
+  extend Puppet::Concurrent::ThreadLocalSingleton
+
   # Produces a String representation of the given type.
   # @param t [PAnyType] the type to produce a string form
   # @return [String] the type in string form
@@ -14,14 +18,31 @@ class TypeFormatter
   # @api public
   #
   def self.string(t)
-    @singleton.string(t)
+    singleton.string(t)
   end
 
-  # @return [TypeCalculator] the singleton instance
-  #
-  # @api private
-  def self.singleton
-    @singleton
+  def initialize
+    @string_visitor = Visitor.new(nil, 'string',0,0)
+  end
+
+  def expanded
+    tf = clone
+    tf.instance_variable_set(:@expanded, true)
+    tf
+  end
+
+  def indented(indent = 0, indent_width = 2)
+    tf = clone
+    tf.instance_variable_set(:@indent, indent)
+    tf.instance_variable_set(:@indent_width, indent_width)
+    tf
+  end
+
+  def ruby(ref_ctor)
+    tf = clone
+    tf.instance_variable_set(:@ruby, true)
+    tf.instance_variable_set(:@ref_ctor, ref_ctor)
+    tf
   end
 
   # Produces a string representing the type
@@ -42,17 +63,24 @@ class TypeFormatter
   #
   # @api public
   def indented_string(t, indent = 0, indent_width = 2)
+    @bld = ''
+    append_indented_string(t, indent, indent_width)
+    @bld
+  end
+
+  # @api private
+  def append_indented_string(t, indent = 0, indent_width = 2, skip_initial_indent = false)
+    save_indent = @indent
+    save_indent_width = @indent_width
     @indent = indent
     @indent_width = indent_width
     begin
-      @bld = ''
-      (@indent * @indent_width).times { @bld << ' ' }
+      (@indent * @indent_width).times { @bld << ' ' } unless skip_initial_indent
       append_string(t)
       @bld << "\n"
-      @bld
     ensure
-      @indent = nil
-      @indent_width = nil
+      @indent = save_indent
+      @indent_width = save_indent_width
     end
   end
 
@@ -77,14 +105,14 @@ class TypeFormatter
     if @ruby && t.is_a?(PAnyType)
       @ruby = false
       begin
-        @bld << @ref_ctor << "('"
-        @@string_visitor.visit_this_0(self, t)
-        @bld << "')"
+        @bld << @ref_ctor << '('
+        @string_visitor.visit_this_0(self, TypeFormatter.new.string(t))
+        @bld << ')'
       ensure
         @ruby = true
       end
     else
-      @@string_visitor.visit_this_0(self, t)
+      @string_visitor.visit_this_0(self, t)
     end
   end
 
@@ -122,13 +150,15 @@ class TypeFormatter
   def string_PDefaultType(_) ; @bld << 'Default' ; end
 
   # @api private
-  def string_PBooleanType(_) ; @bld << 'Boolean' ; end
+  def string_PBooleanType(t)
+    append_array('Boolean', t.value.nil?) { append_string(t.value) }
+  end
 
   # @api private
   def string_PScalarType(_)  ; @bld << 'Scalar'  ; end
 
   # @api private
-  def string_PDataType(_)    ; @bld << 'Data'    ; end
+  def string_PScalarDataType(_)  ; @bld << 'ScalarData'  ; end
 
   # @api private
   def string_PNumericType(_) ; @bld << 'Numeric' ; end
@@ -142,10 +172,15 @@ class TypeFormatter
   end
 
   # @api private
-  def string_PType(t)
+  def string_PTypeType(t)
     append_array('Type', t.type.nil?) { append_string(t.type) }
   end
 
+  # @api private
+  def string_PInitType(t)
+    append_array('Init', t.type.nil?)  { append_strings([t.type, *t.init_args]) }
+  end
+
   # @api private
   def string_PIterableType(t)
     append_array('Iterable', t.element_type.nil?)  { append_string(t.element_type) }
@@ -169,11 +204,10 @@ class TypeFormatter
   # @api private
   def string_PStringType(t)
     range = range_array_part(t.size_type)
-    append_array('String', range.empty?) do
+    append_array('String', range.empty? && !(@debug && !t.value.nil?)) do
       if @debug
-        append_elements(range, true)
-        append_strings(t.values, true)
-        chomp_list
+        append_elements(range, !t.value.nil?)
+        append_string(t.value) unless t.value.nil?
       else
         append_elements(range)
       end
@@ -182,7 +216,13 @@ class TypeFormatter
 
   # @api private
   def string_PEnumType(t)
-    append_array('Enum', t.values.empty?) { append_strings(t.values) }
+    append_array('Enum', t.values.empty?) do
+      append_strings(t.values)
+      if t.case_insensitive?
+        @bld << COMMA_SEP
+        append_string(true)
+      end
+    end
   end
 
   # @api private
@@ -291,12 +331,61 @@ class TypeFormatter
     append_array('Pattern', t.patterns.empty?) { append_strings(t.patterns.map(&:regexp)) }
   end
 
+
   # @api private
   def string_PCollectionType(t)
     range = range_array_part(t.size_type)
     append_array('Collection', range.empty? ) { append_elements(range) }
   end
 
+  def string_Object(t)
+    type = TypeCalculator.infer(t)
+    if type.is_a?(PObjectTypeExtension)
+      type = type.base_type
+    end
+    if type.is_a?(PObjectType)
+      init_hash = type.extract_init_hash(t)
+      @bld << type.name << '('
+      if @indent
+        append_indented_string(init_hash, @indent, @indent_width, true)
+        @bld.chomp!
+      else
+        append_string(init_hash)
+      end
+      @bld << ')'
+    else
+      @bld << 'Instance of '
+      append_string(type)
+    end
+  end
+
+  def string_PuppetObject(t)
+    @bld << t._pcore_type.name << '('
+    if @indent
+      append_indented_string(t._pcore_init_hash, @indent, @indent_width, true)
+      @bld.chomp!
+    else
+      append_string(t._pcore_init_hash)
+    end
+    @bld << ')'
+  end
+
+  # @api private
+  def string_PURIType(t)
+    append_array('URI', t.parameters.nil?) { append_string(t._pcore_init_hash['parameters']) }
+  end
+
+  def string_URI(t)
+    @bld << 'URI('
+    if @indent
+      append_indented_string(t.to_s, @indent, @indent_width, true)
+      @bld.chomp!
+    else
+      append_string(t.to_s)
+    end
+    @bld << ')'
+  end
+
   # @api private
   def string_PUnitType(_)
     @bld << 'Unit'
@@ -304,7 +393,7 @@ class TypeFormatter
 
   # @api private
   def string_PRuntimeType(t)
-    append_array('Runtime') { append_strings([t.runtime, t.name_or_pattern]) }
+    append_array('Runtime', t.runtime.nil? && t.name_or_pattern.nil?) { append_strings([t.runtime, t.name_or_pattern]) }
   end
 
   # @api private
@@ -312,7 +401,7 @@ class TypeFormatter
     if t.has_empty_range?
       append_array('Array') { append_strings([0, 0]) }
     else
-      append_array('Array', t == PArrayType::DATA) do
+      append_array('Array', t == PArrayType::DEFAULT) do
         append_strings([t.element_type], true)
         append_elements(range_array_part(t.size_type), true)
         chomp_list
@@ -325,8 +414,8 @@ class TypeFormatter
     if t.has_empty_range?
       append_array('Hash') { append_strings([0, 0]) }
     else
-      append_array('Hash', t == PHashType::DATA) do
-        append_strings([t.key_type, t.element_type], true)
+      append_array('Hash', t == PHashType::DEFAULT) do
+        append_strings([t.key_type, t.value_type], true)
         append_elements(range_array_part(t.size_type), true)
         chomp_list
       end
@@ -339,7 +428,7 @@ class TypeFormatter
   end
 
   # @api private
-  def string_PHostClassType(t)
+  def string_PClassType(t)
     append_array('Class', t.class_name.nil?) { append_elements([t.class_name]) }
   end
 
@@ -356,8 +445,8 @@ class TypeFormatter
   def string_PNotUndefType(t)
     contained_type = t.type
     append_array('NotUndef', contained_type.nil? || contained_type.class == PAnyType) do
-      if contained_type.is_a?(PStringType) && contained_type.values.size == 1
-        append_string(contained_type.values[0])
+      if contained_type.is_a?(PStringType) && !contained_type.value.nil?
+        append_string(contained_type.value)
       else
         append_string(contained_type)
       end
@@ -366,7 +455,7 @@ class TypeFormatter
 
   # @api private
   def string_PAnnotatedMember(m)
-    hash = m.i12n_hash
+    hash = m._pcore_init_hash
     if hash.size == 1
       string(m.type)
     else
@@ -384,7 +473,7 @@ class TypeFormatter
   # @api private
   def string_PTypeSetType(t)
     append_array('TypeSet') do
-      append_hash(t.i12n_hash.each, proc { |k| @bld << symbolic_key(k) }) do |k,v|
+      append_hash(t._pcore_init_hash.each, proc { |k| @bld << symbolic_key(k) }) do |k,v|
         case k
         when KEY_TYPES
           old_ts = @type_set
@@ -412,23 +501,34 @@ class TypeFormatter
   # @api private
   def string_PObjectType(t)
     if @expanded
-      append_object_hash(t.i12n_hash(@type_set.nil? || !@type_set.defines_type?(t)))
+      append_object_hash(t._pcore_init_hash(@type_set.nil? || !@type_set.defines_type?(t)))
     else
-      @bld << (@type_set ? @type_set.name_for(t) : t.label)
+      @bld << (@type_set ? @type_set.name_for(t, t.label) : t.label)
+    end
+  end
+
+  def string_PObjectTypeExtension(t)
+    append_array(@type_set ? @type_set.name_for(t, t.name) : t.name, false) do
+      ips = t.init_parameters
+      if ips.is_a?(Array)
+        append_strings(ips)
+      else
+        append_string(ips)
+      end
     end
   end
 
   # @api private
   def string_PSensitiveType(t)
-    append_array('Sensitive') { append_string(t.type) }
+    append_array('Sensitive', PAnyType::DEFAULT == t.type) { append_string(t.type) }
   end
 
   # @api private
   def string_POptionalType(t)
     optional_type = t.optional_type
     append_array('Optional', optional_type.nil?) do
-      if optional_type.is_a?(PStringType) && optional_type.values.size == 1
-        append_string(optional_type.values[0])
+      if optional_type.is_a?(PStringType) && !optional_type.value.nil?
+        append_string(optional_type.value)
       else
         append_string(optional_type)
       end
@@ -440,11 +540,17 @@ class TypeFormatter
     expand = @expanded
     if expand && t.self_recursion?
       @guard ||= RecursionGuard.new
-      expand = (@guard.add_this(t) & RecursionGuard::SELF_RECURSION_IN_THIS) == 0
+      @guard.with_this(t) { |state| format_type_alias_type(t, (state & RecursionGuard::SELF_RECURSION_IN_THIS) == 0) }
+    else
+      format_type_alias_type(t, expand)
     end
+  end
+
+  # @api private
+  def format_type_alias_type(t, expand)
     if @type_set.nil?
       @bld << t.name
-      if expand
+      if expand && !Loader::StaticLoader::BUILTIN_ALIASES.include?(t.name)
         @bld << ' = '
         append_string(t.resolved_type)
       end
@@ -452,7 +558,7 @@ class TypeFormatter
       if expand && @type_set.defines_type?(t)
         append_string(t.resolved_type)
       else
-        @bld << @type_set.name_for(t)
+        @bld << @type_set.name_for(t, t.name)
       end
     end
   end
@@ -464,7 +570,17 @@ class TypeFormatter
 
   # @api private
   def string_Array(t)
-    append_array('') { append_strings(t) }
+    append_array('') do
+      if @indent && !is_short_array?(t)
+        @indent += 1
+        t.each { |elem| newline; append_string(elem); @bld << COMMA_SEP }
+        chomp_list
+        @indent -= 1
+        newline
+      else
+        append_strings(t)
+      end
+    end
   end
 
   # @api private
@@ -481,13 +597,13 @@ class TypeFormatter
   end
 
   # @api private
-  def string_NilClass(t)     ; @bld << (@ruby ? 'nil' : '?') ; end
+  def string_NilClass(t)     ; @bld << (@ruby ? 'nil' : 'undef') ; end
 
   # @api private
   def string_Numeric(t)      ; @bld << t.to_s    ; end
 
   # @api private
-  def string_Regexp(t)       ; @bld << t.inspect; end
+  def string_Regexp(t)       ; @bld << PRegexpType.regexp_to_s_with_delimiters(t); end
 
   # @api private
   def string_String(t)
@@ -549,8 +665,29 @@ class TypeFormatter
 
   HASH_ENTRY_OP = ' => '.freeze
 
+  def is_short_array?(t)
+    t.empty? || 100 - @indent * @indent_width > t.inject(0) do |sum, elem|
+      case elem
+      when true, false, nil, Numeric, Symbol
+        sum + elem.inspect.length()
+      when String
+        sum + 2 + elem.length
+      when Hash, Array
+        sum + (elem.empty? ? 2 : 1000)
+      else
+        sum + 1000
+      end
+    end
+  end
+
   def range_array_part(t)
-    t.nil? || t.unbounded? ? EMPTY_ARRAY : [t.from.nil? ? 'default' : t.from.to_s , t.to.nil? ? 'default' : t.to.to_s ]
+    if t.nil? || t.unbounded?
+      EMPTY_ARRAY
+    else
+      result = [t.from.nil? ? 'default' : t.from.to_s]
+      result << t.to.to_s unless t.to.nil?
+      result
+    end
   end
 
   def append_object_hash(hash)
@@ -658,9 +795,6 @@ class TypeFormatter
   def chomp_list
     @bld.chomp!(COMMA_SEP)
   end
-
-  @singleton = new
-  @@string_visitor = Visitor.new(nil, 'string',0,0)
 end
 end
 end
